@@ -85,10 +85,14 @@ class CCodeWriter:
         try:
             return self.input_file_contents[file]
         except KeyError:
-            F = open(file).readlines()
+            F = [line.replace('*/', '*[inserted by cython to avoid comment closer]/')
+                 for line in open(file).readlines()]
             self.input_file_contents[file] = F
             return F
-    
+
+    def get_py_version_hex(self, pyversion):
+        return "0x%02X%02X%02X%02X" % (tuple(pyversion) + (0,0,0,0))[:4]
+
     def mark_pos(self, pos):
         file, line, col = pos
         contents = self.file_contents(file)
