@@ -493,7 +493,7 @@ def p_opt_string_literal(s):
 
 def p_string_literal(s):
     # A single string or char literal.
-    # Returns (kind, value) where kind in ('', 'c', 'r')
+    # Returns (kind, value) where kind in ('', 'c', 'r', 'u')
     if s.sy == 'STRING':
         value = unquote(s.systring)
         s.next()
@@ -502,7 +502,7 @@ def p_string_literal(s):
     pos = s.position()
     #is_raw = s.systring[:1].lower() == "r"
     kind = s.systring[:1].lower()
-    if kind not in "cr":
+    if kind not in "cru":
         kind = ''
     chars = []
     while 1:
@@ -513,6 +513,8 @@ def p_string_literal(s):
             systr = s.systring
             if len(systr) == 1 and systr in "'\"\n":
                 chars.append('\\')
+            if kind == 'u' and not isinstance(systr, unicode):
+                systr = systr.decode("UTF-8")
             chars.append(systr)
         elif sy == 'ESCAPE':
             systr = s.systring
@@ -533,6 +535,8 @@ def p_string_literal(s):
                     chars.append('\\x0' + systr[2:])
                 elif c == '\n':
                     pass
+                elif c == 'u':
+                    chars.append(systr)
                 else:
                     chars.append(r'\\' + systr[1:])
         elif sy == 'NEWLINE':
@@ -546,7 +550,10 @@ def p_string_literal(s):
                 "Unexpected token %r:%r in string literal" %
                     (sy, s.systring))
     s.next()
-    value = join(chars, '')
+    if kind == 'u':
+        value = u''.join(chars)
+    else:
+        value = ''.join(chars)
     #print "p_string_literal: value =", repr(value) ###
     return kind, value
 
@@ -1202,13 +1209,21 @@ def p_statement(s, level, cdef_flag = 0, visibility = 'private'):
         if level not in ('module', 'module_pxd'):
             s.error("ctypedef statement not allowed here")
         return p_ctypedef_statement(s, level, visibility)
+    overridable = 0
     if s.sy == 'cdef':
         cdef_flag = 1
         s.next()
+    if s.sy == 'rdef':
+        cdef_flag = 1
+        overridable = 1
+        s.next()
     if cdef_flag:
         if level not in ('module', 'module_pxd', 'function', 'c_class', 'c_class_pxd'):
             s.error('cdef statement not allowed here')
-        return p_cdef_statement(s, level, visibility)
+        return p_cdef_statement(s, level, visibility, overridable = overridable)
+#    elif s.sy == 'rdef':
+#        s.next()
+#        return p_c_func_or_var_declaration(s, level, s.position(), visibility = visibility, overridable = True)
     elif s.sy == 'def':
         if level not in ('module', 'class', 'c_class', 'property'):
             s.error('def statement not allowed here')
@@ -1227,8 +1242,6 @@ def p_statement(s, level, cdef_flag = 0, visibility = 'private'):
         if level in ('c_class', 'c_class_pxd'):
             if s.sy == 'pass':
                 return p_pass_statement(s, with_newline = 1)
-            else:
-                s.error("Executable statement not allowed here")
         if s.sy == 'if':
             return p_if_statement(s)
         elif s.sy == 'while':
@@ -1522,15 +1535,15 @@ def p_c_func_options(s):
 c_arg_list_terminators = ('*', '**', '.', ')')
 c_arg_list_trailers = ('.', '*', '**')
 
-def p_c_arg_list(s, in_pyfunc, cmethod_flag = 0):
+def p_c_arg_list(s, in_pyfunc, cmethod_flag = 0, kw_only = 0):
     args = []
     if s.sy not in c_arg_list_terminators:
-        args.append(p_c_arg_decl(s, in_pyfunc, cmethod_flag))
+        args.append(p_c_arg_decl(s, in_pyfunc, cmethod_flag, kw_only))
         while s.sy == ',':
             s.next()
             if s.sy in c_arg_list_terminators:
                 break
-            args.append(p_c_arg_decl(s, in_pyfunc))
+            args.append(p_c_arg_decl(s, in_pyfunc, kw_only = kw_only))
     return args
 
 def p_optional_ellipsis(s):
@@ -1540,7 +1553,7 @@ def p_optional_ellipsis(s):
     else:
         return 0
 
-def p_c_arg_decl(s, in_pyfunc, cmethod_flag = 0):
+def p_c_arg_decl(s, in_pyfunc, cmethod_flag = 0, kw_only = 0):
     pos = s.position()
     not_none = 0
     default = None
@@ -1562,10 +1575,13 @@ def p_c_arg_decl(s, in_pyfunc, cmethod_flag = 0):
         base_type = base_type,
         declarator = declarator,
         not_none = not_none,
-        default = default)
+        default = default,
+        kw_only = kw_only)
 
-def p_cdef_statement(s, level, visibility = 'private'):
+def p_cdef_statement(s, level, visibility = 'private', overridable = False):
     pos = s.position()
+    if overridable and level not in ('c_class', 'c_class_pxd'):
+            error(pos, "Overridable cdef function not allowed here")
     visibility = p_visibility(s, visibility)
     if visibility == 'extern' and s.sy in ('from' ,':'):
             return p_cdef_extern_block(s, level, pos)
@@ -1587,7 +1603,7 @@ def p_cdef_statement(s, level, visibility = 'private'):
         s.expect_newline('Expected a newline')
         return node
     else:
-        return p_c_func_or_var_declaration(s, level, pos, visibility)
+        return p_c_func_or_var_declaration(s, level, pos, visibility, overridable)
 
 def p_cdef_extern_block(s, level, pos):
     include_file = None
@@ -1692,13 +1708,13 @@ def p_visibility(s, prev_visibility):
     return visibility
     
 def p_c_modifiers(s):
-    if s.systring in ('inline', ):
-        modifier = s.systring.upper() # uppercase is macro defined for various compilers
+    if s.sy == 'IDENT' and s.systring in ('inline',):
+        modifier = s.systring
         s.next()
-        return modifier + ' ' + p_c_modifiers(s)
-    return ""
+        return [modifier] + p_c_modifiers(s)
+    return []
 
-def p_c_func_or_var_declaration(s, level, pos, visibility = 'private'):
+def p_c_func_or_var_declaration(s, level, pos, visibility = 'private', overridable = False):
     cmethod_flag = level in ('c_class', 'c_class_pxd')
     modifiers = p_c_modifiers(s)
     base_type = p_c_base_type(s)
@@ -1712,7 +1728,8 @@ def p_c_func_or_var_declaration(s, level, pos, visibility = 'private'):
             base_type = base_type,
             declarator = declarator, 
             body = suite,
-            modifiers = modifiers)
+            modifiers = modifiers,
+            overridable = overridable)
     else:
         if level == 'module_pxd' and visibility <> 'extern':
             error(pos, 
@@ -1764,9 +1781,12 @@ def p_def_statement(s):
     starstar_arg = None
     if s.sy == '*':
         s.next()
-        star_arg = p_py_arg_decl(s)
+        if s.sy == 'IDENT':
+            star_arg = p_py_arg_decl(s)
         if s.sy == ',':
             s.next()
+            if s.sy == 'IDENT':
+                args.extend(p_c_arg_list(s, in_pyfunc = 1, kw_only = 1))
             if s.sy == '**':
                 s.next()
                 starstar_arg = p_py_arg_decl(s)
