@@ -138,7 +138,7 @@ reserved_words = [
     "raise", "import", "exec", "try", "except", "finally",
     "while", "if", "elif", "else", "for", "in", "assert",
     "and", "or", "not", "is", "in", "lambda", "from",
-    "NULL", "cimport", "by", "with"
+    "NULL", "cimport", "by", "with", "rdef"
 ]
 
 function_contexts = [ # allowed arguments to the "with" option
