@@ -5,6 +5,7 @@
 
 import Naming
 import PyrexTypes
+import sys
 
 class Signature:
     #  Method slot signature descriptor.
@@ -104,17 +105,23 @@ class SlotDescriptor:
     #  slot_name    string           Member name of the slot in the type object
     #  is_initialised_dynamically    Is initialised by code in the module init function
 
-    def __init__(self, slot_name, dynamic = 0):
+    def __init__(self, slot_name, dynamic = 0, min_python_version = None):
         self.slot_name = slot_name
         self.is_initialised_dynamically = dynamic
-    
+        self.min_python_version = min_python_version
+
     def generate(self, scope, code):
         if self.is_initialised_dynamically:
             value = 0
         else:
             value = self.slot_code(scope)
+        if self.min_python_version is not None:
+            code.putln("#if PY_VERSION_HEX >= " +
+                       code.get_py_version_hex(self.min_python_version))
         code.putln("%s, /*%s*/" % (value, self.slot_name))
-    
+        if self.min_python_version is not None:
+            code.putln("#endif")
+
     # Some C implementations have trouble statically 
     # initialising a global with a pointer to an extern 
     # function, so we initialise some of the type slots
@@ -157,7 +164,7 @@ class GCDependentSlot(SlotDescriptor):
     #  the type participates in GC.
     
     def __init__(self, slot_name, no_gc_value, gc_value, dynamic = 0):
-        SlotDescriptor.__init__(self, slot_name, dynamic)
+        SlotDescriptor.__init__(self, slot_name, dynamic = dynamic)
         self.no_gc_value = no_gc_value
         self.gc_value = gc_value
     
@@ -175,8 +182,10 @@ class MethodSlot(SlotDescriptor):
     #  method_name  string           The __xxx__ name of the method
     #  default      string or None   Default value of the slot
     
-    def __init__(self, signature, slot_name, method_name, default = None):
-        SlotDescriptor.__init__(self, slot_name)
+    def __init__(self, signature, slot_name, method_name,
+                 default = None, min_python_version = None):
+        SlotDescriptor.__init__(self, slot_name,
+                                min_python_version = min_python_version)
         self.signature = signature
         self.slot_name = slot_name
         self.method_name = method_name
@@ -493,7 +502,7 @@ PyNumberMethods = (
     MethodSlot(ibinaryfunc, "nb_inplace_true_divide", "__itruediv__"),
 
     # Added in release 2.5
-    MethodSlot(unaryfunc, "nb_index", "__index__"),
+    MethodSlot(unaryfunc, "nb_index", "__index__", min_python_version=(2,5)),
 )
 
 PySequenceMethods = (
