@@ -8,10 +8,14 @@ setup(
     Extension("primes",       ["primes.pyx"]),
     Extension("spam",         ["spam.pyx"]),
 #    Extension("numeric_demo", ["numeric_demo.pyx"]),
-    Extension("test", ["test.pyx"]),
+#    Extension("test", ["test.pyx"]),
     Extension("func_pointers", ["func_pointers.pyx"]),
-    Extension("inplace", ["inplace.pyx"]),
+#    Extension("inplace", ["inplace.pyx"]),
     Extension("withGIL", ["withGIL.pyx"]),
+    Extension("class_members", ["class_members.pyx"]),
+#    Extension("inherit_bug", ["inherit_bug.pyx"]),
+    Extension("override", ["override.pyx"]),
+    Extension("cond", ["cond.pyx"]),
     ],
   cmdclass = {'build_ext': build_ext},
 #  include_dirs = "/System/Library/Frameworks/Python.framework/Versions/2.3/include/python2.3/"
